@@ -141,6 +141,9 @@
 		case LIBMTP_FILETYPE_HTML:
 			return @"HTML";
 			break;
+		case LIBMTP_FILETYPE_FIRMWARE:
+			return @"firmware";
+			break;
 	}
 	// we can't ever get here...
 	NSLog(@"invalid libmtp file type %d", fileType);
