@@ -15,12 +15,6 @@
 #import "Directory.h"
 #include "libmtp.h"
 
-#define PLAYLISTS_ASSOC_NAME @"My Playlists"
-#define MUSIC_ASSOC_NAME @"Music"
-#define PICTURES_ASSOC_NAME @"Pictures"
-#define VIDEO_ASSOC_NAME @"Video"
-#define ZENCAST_ASSOC_NAME @"ZENcast"
-
 @interface NJB : NSObject {
 	IBOutlet StatusDisplayer *statusDisplayer;
 	njb_t *njb;
