@@ -1228,7 +1228,9 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NJBTransactionResult *)setBitmap:(NSString *)bitmapPath
 {
-	if (mtpDevice)
+	NSLog(@"setBitmap not implemented!");
+	return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+	/*if (mtpDevice)
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	
 	if (NJB_Set_Bitmap(njb, [bitmapPath UTF8String]) == -1)
@@ -1237,7 +1239,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		NSLog(error);
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
-	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];*/
 }
 
 - (NJBTransactionResult *)setTime:(NSNumber *)timeIntervalSinceNow
