@@ -131,13 +131,17 @@
 				[mainThreadQueue addObject:njbQueueItem];
 				[mainThreadQueueLock unlock];
 				// tell the main thread there are queue items
+				NSLog(@"NJBQueueConsumer telling mainThreadProxy to consume queue");
 				[mainThreadProxy consumeQueue];
+				NSLog(@"mainThreadProxy has returned");
 				if ([njbQueueItem displayStatus])
 					NSLog(@"displayStatus == YES for runInMainThread queue item!");
 			}
 			else
 			{
+				NSLog(@"processing njbQueueItem");
 				id result = [njbQueueItem process];
+				NSLog(@"returned from processing njbQueueItem");
 				if ([njbQueueItem displayStatus])
 				{
 					if ([result class] == [NJBTransactionResult class])
@@ -153,6 +157,8 @@
 			[status setIdle];
 		
 		[pool release];
+		
+		NSLog(@"going round queue loop again...");
 	}
 	// never gets here...
 	NSLog(@"Got to end of run loop!");
