@@ -17,6 +17,7 @@
 	statusTypes status;
 	BOOL displayStatus;
 	BOOL cancelled;
+	NSMutableArray *itemsToCancelIfFail;
 }
 - (void)setTarget:(id)newTarget;
 - (void)setSelector:(SEL)newSelector;
@@ -24,7 +25,7 @@
 - (void)setObject2:(id)newObject2;
 - (void)setStatus:(statusTypes)newStatus;
 - (statusTypes)status;
-- (void)setRunInMainThread:(bool)newRunInMainThread;
+- (void)setRunInMainThread:(BOOL)newRunInMainThread;
 - (BOOL)runInMainThread;
 - (void)setDisplayStatus:(BOOL)newDisplayStatus;
 - (BOOL)displayStatus;
@@ -34,4 +35,6 @@
 - (id)initWithTarget:(id)newTarget withSelector:(SEL)newSelector withObject:(id)newObject withRunInMainThread:(BOOL)newRunInMainThread;
 - (void)cancel;
 - (BOOL)cancelled;
+- (void)cancelItemIfFail:(NJBQueueItem *)item;
+
 @end
