@@ -118,6 +118,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   { "Philips HDD1630/17", 0x0471, 0x014c, DEVICE_FLAG_NONE },
   // From Gerhard Mekenkamp
   { "Philips GoGear Audio", 0x0471, 0x0165, DEVICE_FLAG_NONE },
+  // from XNJB forum
+  { "Philips GoGear SA9200", 0x0471, 0x014f, DEVICE_FLAG_NONE },
 
   /*
    * SanDisk
@@ -222,9 +224,9 @@ static struct usb_device* find_device (int busn, int devicen, short force);
 static void find_endpoints(struct usb_device *dev, int* inep, int* inep_maxpacket, int* outep, int* outep_maxpacket, int* intep);
 static void clear_stall(PTP_USB* ptp_usb);
 static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device* dev);
-static short ptp_write_func (unsigned char *bytes, unsigned int size, void *data);
-static short ptp_read_func (unsigned char *bytes, unsigned int size, void *data, unsigned int *readbytes);
-static short ptp_check_int (unsigned char *bytes, unsigned int size, void *data, unsigned int *rlen);
+static short ptp_write_func (unsigned long,PTPDataHandler*,void *data,unsigned long*);
+static short ptp_read_func (unsigned long,PTPDataHandler*,void *data,unsigned long*);
+static short ptp_check_int (unsigned long,PTPDataHandler*,void *, unsigned long *);
 static int usb_clear_stall_feature(PTP_USB* ptp_usb, int ep);
 static int usb_get_endpoint_status(PTP_USB* ptp_usb, int ep, uint16_t* status);
 
@@ -396,37 +398,62 @@ void dump_usbinfo(PTP_USB *ptp_usb)
 }
 
 
-// Based on same function on library.c in libgphoto2
-#define CONTEXT_BLOCK_SIZE	0x100000
+/*
+ * ptp_read_func() and ptp_write_func() are
+ * based on same functions in library.c in libgphoto2.
+ * Much reading packet logs and having fun with trials and errors
+ * reveals that WMP / Windows is probably using an algorithm like this
+ * for large transfers:
+ *
+ * 1. Send the command (0x0c bytes) if headers are split, else, send 
+ *    command plus sizeof(endpoint) - 0x0c bytes.
+ * 2. Send first packet, max size to be sizeof(endpoint) but only when using
+ *    split headers. Else goto 3.
+ * 3. REPEAT send 0x10000 byte chunks UNTIL remaining bytes < 0x10000
+ *    We call 0x10000 CONTEXT_BLOCK_SIZE.
+ * 4. Send remaining bytes MOD sizeof(endpoint)
+ * 5. Send remaining bytes. If this happens to be exactly sizeof(endpoint)
+ *    then also send a zero-length package.
+ */
+#define CONTEXT_BLOCK_SIZE	0x10000
 static short
-ptp_read_func (unsigned char *bytes, unsigned int size, void *data, unsigned int *readbytes)
-{
+ptp_read_func (
+	unsigned long size, PTPDataHandler *handler,void *data,
+	unsigned long *readbytes
+) {
   PTP_USB *ptp_usb = (PTP_USB *)data;
-  int toread = 0;
+  unsigned long toread = 0;
   int result = 0;
-  int curread = 0;
-  /* Split into small blocks. Too large blocks (>1x MB) would
-   * timeout.
-   */
+  unsigned long curread = 0;
+  unsigned long written;
+  unsigned char *bytes;
+
+  // This is the largest block we'll need to read in.
+  bytes = malloc(CONTEXT_BLOCK_SIZE);
   while (curread < size) {
     toread = size - curread;
     if (toread > CONTEXT_BLOCK_SIZE)
       toread = CONTEXT_BLOCK_SIZE;
-    
-    result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep,(char *)(bytes+curread), toread, ptpcam_usb_timeout);
+    else if (toread > ptp_usb->outep_maxpacket)
+      toread -= toread % ptp_usb->outep_maxpacket;
+
+    result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, (char*)bytes, toread, ptpcam_usb_timeout);
     if (result == 0) {
-      result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep,(char *)(bytes+curread), toread, ptpcam_usb_timeout);
+      result = USB_BULK_READ(ptp_usb->handle, ptp_usb->inep, (char*)bytes, toread, ptpcam_usb_timeout);
     }
     if (result < 0)
       return PTP_ERROR_IO;
 #ifdef ENABLE_USB_BULK_DEBUG
     printf("<==USB IN\n");
-    data_dump_ascii (stdout,(bytes+curread),result,16);
+    data_dump_ascii (stdout,bytes,result,16);
 #endif
+    handler->putfunc (NULL, handler->private, result, bytes, &written);
     curread += result;
     if (result < toread) /* short reads are common */
       break;
   }
+  if (readbytes) *readbytes = curread;
+  free (bytes);
 
   // Increase counters, call callback
   if (ptp_usb->callback_active) {
@@ -445,34 +472,40 @@ ptp_read_func (unsigned char *bytes, unsigned int size, void *data, unsigned int
   }
   
   if (result > 0) {
-    *readbytes = curread;
     return (PTP_RC_OK);
   } else {
     return PTP_ERROR_IO;
   }
 }
 
-// Based on same function on library.c in libgphoto2
 static short
-ptp_write_func (unsigned char *bytes, unsigned int size, void *data)
-{
+ptp_write_func (
+        unsigned long   size,
+        PTPDataHandler  *handler,
+        void            *data,
+        unsigned long   *written
+) {
   PTP_USB *ptp_usb = (PTP_USB *)data;
-  int towrite = 0;
+  unsigned long towrite = 0;
   int result = 0;
-  int curwrite = 0;
-  
-  /*
-   * gp_port_write returns (in case of success) the number of bytes
-   * written. Too large blocks (>5x MB) could timeout.
-   */
+  unsigned long curwrite = 0;
+  unsigned char *bytes;
+
+  // This is the largest block we'll need to read in.  
+  bytes = malloc(CONTEXT_BLOCK_SIZE);
+  if (!bytes) return PTP_ERROR_IO;
   while (curwrite < size) {
     towrite = size-curwrite;
     if (towrite > CONTEXT_BLOCK_SIZE)
       towrite = CONTEXT_BLOCK_SIZE;
-    result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)(bytes+curwrite),towrite,ptpcam_usb_timeout);
+    else
+      if (towrite > ptp_usb->outep_maxpacket && towrite % ptp_usb->outep_maxpacket != 0)
+        towrite -= towrite % ptp_usb->outep_maxpacket;
+    handler->getfunc (NULL, handler->private,towrite,bytes,&towrite);
+    result = USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char*)bytes,towrite,ptpcam_usb_timeout);
 #ifdef ENABLE_USB_BULK_DEBUG
     printf("USB OUT==>\n");
-    data_dump_ascii (stdout,(bytes+curwrite),towrite,16);
+    data_dump_ascii (stdout,bytes,towrite,16);
 #endif
     if (result < 0)
       return PTP_ERROR_IO;
@@ -480,10 +513,14 @@ ptp_write_func (unsigned char *bytes, unsigned int size, void *data)
     if (result < towrite) /* short writes happen */
       break;
   }
+  free (bytes);
+  if (written) *written = curwrite;
 
-  // Increase counters, call callback
+  // Increase counters
+  ptp_usb->current_transfer_complete += curwrite;
+  
+  // call callback
   if (ptp_usb->callback_active) {
-    ptp_usb->current_transfer_complete += curwrite;
     if (ptp_usb->current_transfer_complete > ptp_usb->current_transfer_total) {
       // Fishy... but some commands have unpredictable lengths.
       // send last update and disable callback.
@@ -497,47 +534,41 @@ ptp_write_func (unsigned char *bytes, unsigned int size, void *data)
     }
   }
   
-  // If this is the last transfer (callbacks only active if this function called repeatedly with
-  // new data, otherwise this is a single large transaction which ends here).
-  // a request and the bulk send header.
-  if (!ptp_usb->callback_active) {
-    // Then terminate an even packet boundary write with a zero length packet
-    if ((size % ptp_usb->outep_maxpacket) == 0) {
+  // If this is the last transfer send a zero write if required
+  if (ptp_usb->current_transfer_complete >= ptp_usb->current_transfer_total) {
+    if ((towrite % ptp_usb->outep_maxpacket) == 0) {
+#ifdef ENABLE_USB_BULK_DEBUG
+      printf("USB OUT==>\n");
+      printf("Zero Write\n");
+#endif
       result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)"x",0,ptpcam_usb_timeout);
     }
-  } else if (ptp_usb->current_transfer_complete == ptp_usb->current_transfer_total) {
-    // This is the amount actually transfered in this large transaction.
-    uint64_t actual_xfer_size = ptp_usb->current_transfer_total - 2*PTP_USB_BULK_HDR_LEN;
-    
-    if ((actual_xfer_size % ptp_usb->outep_maxpacket) == 0) {
-      result=USB_BULK_WRITE(ptp_usb->handle,ptp_usb->outep,(char *)"x",0,ptpcam_usb_timeout);
-    }
-    // Set as complete and disable callback, just as good. 
-    // This also blocks callbacks from the following response command.
-    if (ptp_usb->current_transfer_callback != NULL) {
-      (void) ptp_usb->current_transfer_callback(ptp_usb->current_transfer_total, // Both total = 100%
-						ptp_usb->current_transfer_total, // Both total = 100%
-						ptp_usb->current_transfer_callback_data);
-    }
-    ptp_usb->callback_active = 0;
   }
-
+  
+  if (ptp_usb->current_transfer_complete == ptp_usb->current_transfer_total)
+    ptp_usb->callback_active = 0;
+  
   if (result < 0)
     return PTP_ERROR_IO;
   return PTP_RC_OK;
 }
 
 // This is a bit hackish at the moment. I wonder if it even works.
-static short ptp_check_int (unsigned char *bytes, unsigned int size, void *data, unsigned int *rlen)
-{
+static short ptp_check_int (
+	unsigned long size, PTPDataHandler *handler,void *data,
+	unsigned long *readbytes
+) {
   PTP_USB *ptp_usb = (PTP_USB *)data;
   int result;
+  unsigned char bytes[64];
+
+  if (size>64) return PTP_ERROR_IO;
   
   result=USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *)bytes,size,ptpcam_usb_timeout);
   if (result==0)
     result = USB_BULK_READ(ptp_usb->handle, ptp_usb->intep,(char *) bytes, size, ptpcam_usb_timeout);
   if (result >= 0) {
-    *rlen = result;
+    handler->putfunc (NULL, handler->private, result, bytes, readbytes);
     return (PTP_RC_OK);
   } else {
     return PTP_ERROR_IO;
@@ -582,7 +613,7 @@ static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device*
     }
 #endif
 #ifdef __WIN32__
-    // Only needed on Windows
+    // Only needed on Windows, and cause problems on other platforms.
     if (usb_set_configuration(device_handle, dev->config->bConfigurationValue)) {
       perror("usb_set_configuration()");
       return -1;
